@@ -3,16 +3,15 @@ import itertools
 import logging
 import time
 
-import numpy as np
-import matplotlib.pyplot as plt
-import pandas as pd
-from sklearn.metrics import confusion_matrix, cohen_kappa_score
 import fiona
+import matplotlib.pyplot as plt
+import numpy as np
+import pandas as pd
 import rasterio
 from rasterio import mask
 from shapely.geometry import shape
 from shapely.ops import cascaded_union
-
+from sklearn.metrics import cohen_kappa_score, confusion_matrix
 
 ACCURACY_LOGGER = logging.getLogger(__name__)
 
@@ -90,7 +89,7 @@ def compute_confusion_matrix(y_test, preds, cm_fn, plot=True, csv=True):
             plot_confusion_matrix(conf_matrix,
                                   classes=cm_labels,
                                   normalize=True
-                                 )
+                                  )
             plt.savefig(cm_fn + '_plot.png', dpi=150)
         except AttributeError:  # Not all models have FIs, so skip
             pass
@@ -106,7 +105,7 @@ def plot_confusion_matrix(conf_matrix, classes,
     """
     if normalize:
         conf_matrix = conf_matrix.astype('float') /\
-                      conf_matrix.sum(axis=1)[:, np.newaxis]
+            conf_matrix.sum(axis=1)[:, np.newaxis]
     if labeldict is not None:
         classes = map(labeldict.get, classes)
     plt.imshow(conf_matrix, interpolation='nearest', cmap=cmap)
@@ -222,24 +221,9 @@ def assess_accuracy(raster, rois, cm_fn, subset=None):
             len(shapefile),
             counter)
 
-    #write confusion matrix
+    # write confusion matrix
     compute_confusion_matrix(
         sample_labels, all_samples, cm_fn, plot=True, csv=False)
 
     ACCURACY_LOGGER.info(
         "Total run time was %i seconds", (int(time.time() - start)))
-
-
-if __name__ == "__main__":
-    START = time.time()
-    RASTER = "classification.tif"
-    ROIS = "merged_gewassen.geojson"
-    SUBSET = "subset_merged_gewassen.geojson"
-    CM_FN = "confusion_matrix_total"
-
-    assess_accuracy(RASTER, ROIS, CM_FN, SUBSET)
-
-    ACCURACY_LOGGER.info(
-        "Total run time was %i seconds",
-        (int(time.time() - START))
-    )
