@@ -2,22 +2,25 @@
 
 import logging
 import os
-import time
 import shutil
 import sys
+import time
+
 
 import fiona
 import pandas as pd
 
+from classifier.accuracy import (plot_feature_importances,
+                                 write_confusion_matrix)
 from classifier.dataprep import createdataset, outlier_removal
-from classifier.train import train_dataset
-from classifier.accuracy import write_confusion_matrix, plot_feature_importances
 from classifier.predict import prediction
-from classifier.unsupervised import make_input_array, train_kmeans
-from classifier.utils.general import save_model, read_model, parallel_function
-from classifier.utils.raster import stitch, get_meta
+from classifier.settings import US_ALGORITHMS, WORKSPACE
 from classifier.timeseries import get_timeseries_samples
-from classifier.settings import WORKSPACE, US_ALGORITHMS
+from classifier.train import train_dataset
+from classifier.unsupervised import make_input_array, train_kmeans
+from classifier.utils.general import parallel_function, read_model, save_model
+from classifier.utils.raster import get_meta, stitch
+from classifier.utils.config import Configuration
 
 START = time.time()
 CLASSIFIER_LOGGER = logging.getLogger(__name__)
@@ -39,6 +42,7 @@ def read_samples(samples, remove_outliers=True):
         sample_df = outlier_removal(sample_df)
     return sample_df
 
+
 def get_rois_extent(rois):
     """
 
@@ -52,14 +56,15 @@ def get_rois_extent(rois):
     with fiona.open(rois, "r") as shapefile:
         return shapefile.bounds
 
-def gather_samples(rois, rasters, out_dir, config_dict):
+
+def gather_samples(rois, rasters, out_dir, config):
     """Gather samples from a rois file and raster files
 
     Args:
         rois: The path to a rois file
         rasters: A list of raster files
         out_dir: Path where to write the dataset file
-        config_dict: The complete classifier configuration dictionary
+        config (Configuration) contains config
 
     Returns:
         A dataframe containing pixel values and classes
@@ -68,21 +73,21 @@ def gather_samples(rois, rasters, out_dir, config_dict):
     # Train a model and continue
     samples = createdataset(
         rasters,
-        os.path.join(WORKSPACE, rois),
+        rois,
         out_dir,
-        config_dict)
+        config)
     samples.to_csv(os.path.join(out_dir, 'samples.csv'))
     return samples
 
 
-def train(samples, rasters, out_dir, config_dict, rois_extent):
+def train(samples, rasters, out_dir, config, rois_extent):
     """Train a model using samples and rasters
 
     Args:
         samples: Samples dataframe containing pixel values and class values
         rasters: A list of raster files
         out_dir: Path where to write the dataset file
-        config_dict: The complete classifier configuration dictionary
+        config (Configuration) contains config
         rois_extent: The extent of the training data (if any, else None)
 
     Returns:
@@ -91,25 +96,25 @@ def train(samples, rasters, out_dir, config_dict, rois_extent):
     """
     CLASSIFIER_LOGGER.info("\n####-----Training----#####\n")
     dataset = samples
-    windows, _ = get_meta(rasters, config_dict['app_window'])
-    if config_dict['app_algorithm'] in US_ALGORITHMS and len(samples) < 1:
+    windows, _ = get_meta(rasters, config.app.window)
+    if config.app.algorithm in US_ALGORITHMS and len(samples) < 1:
         # Do unsupervised
         array = make_input_array(
             rasters,
             windows,
-            config_dict
+            config
         )
         model_dict = train_kmeans(array,
-                                  config_dict)
+                                  config)
     else:  # All supervised methods
         model_dict, test = train_dataset(
             dataset,
-            config_dict['app_algorithm'],
+            config.app.algorithm,
             out_dir,
-            config_dict
-            )
-        if config_dict['app_algorithm'] in ['randomforest', 'xgboost'] \
-                and config_dict['acc_perform_assesment']:
+            config
+        )
+        if config.app.algorithm in ['randomforest', 'xgboost'] \
+                and config.accuracy.perform_assesment:
             # Do the accuracy analysis
             cm_fn = os.path.join(out_dir, 'confusion_matrix')
             fi_fn = os.path.join(out_dir, 'feature_importance.png')
@@ -118,25 +123,25 @@ def train(samples, rasters, out_dir, config_dict, rois_extent):
     # save the model as a python pickle
     if not rois_extent is None:
         model_dict['rois_bounds'] = rois_extent
-    if config_dict['model_save']:
-        save_model(model_dict, out_dir, config_dict)
+    if config.app.model_save:
+        save_model(model_dict, out_dir, config)
     CLASSIFIER_LOGGER.info("\nFinished Training\n")
     return model_dict
 
 
-def predict(model_dict, rasters, out_dir, config_dict):
+def predict(model_dict, rasters, out_dir, config):
     """Prediction function using a trained model and raster files
 
     Args:
         model_dict: A trained model
         rasters: A list of raster files
         out_dir: Path where to write the dataset file
-        config_dict:  The command line arguments dictionary
+        config (Configuration) contains config
 
     """
-    threads = config_dict['app_threads']
+    threads = config.app.threads
     CLASSIFIER_LOGGER.info("\n####-----Prediction----#####\n")
-    windows, meta = get_meta(rasters, config_dict['app_window'])
+    windows, meta = get_meta(rasters, config.app.window)
 
     iterable = [
         {
@@ -144,15 +149,14 @@ def predict(model_dict, rasters, out_dir, config_dict):
             'rasters': rasters,
             'model_dict': model_dict,
             'meta': meta,
-            'config_dict': config_dict
+            'config': config
         }
         for window in windows
     ]
     if threads > 1 or threads == -1:
-
         parallel_function(prediction,
-                              iterable,
-                              threads)
+                          iterable,
+                          threads)
     else:
         for wins in iterable:
             prediction(**wins)
@@ -162,14 +166,14 @@ def predict(model_dict, rasters, out_dir, config_dict):
     CLASSIFIER_LOGGER.info("\n####-----Stitching----#####\n")
 
     # Run the gdalwarp command in the command line
-    stitch(out_dir, meta, config_dict)
+    stitch(out_dir, meta, config)
     CLASSIFIER_LOGGER.info("Cleaning up..")
-    shutil.rmtree(config_dict['tmp_dir'])
+    shutil.rmtree(config.tmp_dir)
     CLASSIFIER_LOGGER.info(
         "Total run time was %i seconds", (int(time.time() - START)))
 
 
-def train_and_predict_with_samples(samples, rasters, out_dir, config_dict,
+def train_and_predict_with_samples(samples, rasters, out_dir, config,
                                    rois_extent=None):
     """Train model using samples
 
@@ -177,17 +181,17 @@ def train_and_predict_with_samples(samples, rasters, out_dir, config_dict,
         samples: Samples dataframe containing pixel values and class values
         rasters: A list of raster files
         out_dir: Path where to write the dataset file
-        config_dict:  The command line arguments dictionary
+        config (Configuration) contains config
 
     Returns:
         The return value. True for success, False otherwise.
 
     """
-    model_dict = train(samples, rasters, out_dir, config_dict, rois_extent)
-    predict(model_dict, rasters, out_dir, config_dict)
+    model_dict = train(samples, rasters, out_dir, config, rois_extent)
+    predict(model_dict, rasters, out_dir, config)
 
 
-def gather_train_predict_rois(rois, rasters, out_dir, config_dict):
+def gather_train_predict_rois(rois, rasters, out_dir, config):
     """Gather samples from rois file and rasters and continue with training and
     prediction
 
@@ -195,60 +199,78 @@ def gather_train_predict_rois(rois, rasters, out_dir, config_dict):
         rois: The path to a rois file
         rasters: A list of raster files
         out_dir: Path where to write the dataset file
-        config_dict:  The command line arguments dictionary
+        config (Configuration) contains config
 
     Returns:
         The return value. True for success, False otherwise.
 
     """
-    samples = gather_samples(rois, rasters, out_dir, config_dict)
+    samples = gather_samples(rois, rasters, out_dir, config)
     rois_extent = get_rois_extent(rois)
     return train_and_predict_with_samples(samples,
                                           rasters,
                                           out_dir,
-                                          config_dict,
+                                          config,
                                           rois_extent)
 
-def classify_(rasters, config_dict, out_dir, rois=None):
+
+def classify_(rasters, config, out_dir, rois=None):
     """Entry point function for pixel-based classification
 
         Args:
             rasters (list): List of raster paths
-            config_dict (dict): CLI configuration dict
+            config (Configuration): CLI configuration dict
             out_dir (str): Path to output directory
             rois (str): Path to rois file
 
     """
-    if not config_dict['app_model'] is None:
-        model = read_model(config_dict['app_model'])
-        predict(model, rasters, out_dir, config_dict)
-    elif not config_dict['app_samples'] is None:
-        samples = read_samples(
-            config_dict['app_samples'],
-            config_dict['su_remove_outliers'])
+    if config.app.model is not None:
+        model = read_model(config.app.model)
+        predict(model, rasters, out_dir, config)
+    if config.app.algorithm not in US_ALGORITHMS:
+        print(config.app.algorithm)
+        # Supervised classification
+        if config.app.samples is not None:
 
-        train_and_predict_with_samples(samples,
-                                       rasters,
-                                       out_dir,
-                                       config_dict)
+            samples = read_samples(
+                config.app.samples,
+                config.supervised.remove_outliers)
 
-    elif rois is not None:
-        gather_train_predict_rois(
-            rois,
-            rasters,
-            out_dir,
-            config_dict)
+            train_and_predict_with_samples(samples,
+                                           rasters,
+                                           out_dir,
+                                           config)
+
+        elif rois is not None:
+            gather_train_predict_rois(
+                rois,
+                rasters,
+                out_dir,
+                config)
+        else:
+            CLASSIFIER_LOGGER.error(
+                "You have chosen a supervised classification method but didn't "
+                "provide rois or samples. Either provide one of these or run a "
+                "unsupervised classification")
+            sys.exit(1)
     else:
-        CLASSIFIER_LOGGER.info("No models, samples or rois provided. Doing "
-                               "unsupervised classification")
-        if not config_dict['app_algorithm'] in US_ALGORITHMS:
-            config_dict['app_algorithm'] = 'us_kmeans'
+        print("eHEEERE")
+        if config.app.samples is not None or rois is not None:
+            CLASSIFIER_LOGGER.error(
+                "You have chosen an unsupervised classification "
+                "method but provided samples or rois. Please choose "
+                "a supervised method to use them or don't provide them for "
+                "an unsupervised classification.")
+            sys.exit(1)
+        CLASSIFIER_LOGGER.info("Doing unsupervised classification")
         train_and_predict_with_samples([],
                                        rasters,
                                        out_dir,
-                                       config_dict)
+                                       config)
+
+
 def classify_timeseries(rasters: list,
-                        config_dict: dict,
+                        config: Configuration,
                         out_dir: str,
                         rois: str):
     """Main function to classifiy timeseries.
@@ -257,28 +279,34 @@ def classify_timeseries(rasters: list,
     or samples does not work.
 
         Args:
-            rasters (list): List of raster paths
-            config_dict (dict): CLI configuration dict
+            rasters (Configuration): contains config
             out_dir (str): Path to output directory
             rois (str): Path to rois file
 
     """
     # Gather samples and do imputation.
     CLASSIFIER_LOGGER.info("Now Getting Samples and doing imputation")
-    samples_df = get_timeseries_samples(rasters, rois, out_dir, config_dict)
+    if rois is None:
+        CLASSIFIER_LOGGER.error(
+            "You have to provide rois for timeseries classification.")
+        sys.exit()
+    rois = os.path.join(WORKSPACE, rois)
+    samples_df = get_timeseries_samples(rasters, rois, out_dir, config)
     # Reshape it
     samples = samples_df.T
     samples['class'] = samples.index
-    if config_dict['app_algorithm'] == 'unsupervised':
-        CLASSIFIER_LOGGER.error("Unsupervised classification not supported yet")
+    if config.app.algorithm in US_ALGORITHMS:
+        CLASSIFIER_LOGGER.error(
+            "Unsupervised classification not supported yet")
         sys.exit()
     model = train(samples,
                   rasters=rasters,
                   out_dir=out_dir,
-                  config_dict=config_dict,
+                  config=config,
                   rois_extent=None)
 
-    predict(model, rasters, out_dir, config_dict)
+    predict(model, rasters, out_dir, config)
+
 
 if __name__ == "__main__":
     pass
