@@ -1,17 +1,18 @@
 "Prediction Module"
 import logging
-from multiprocessing import process
 import os
+from multiprocessing import process
 
 import numpy as np
-from numpy import array
 import rasterio
+from numpy import array
 from rasterio.windows import Window
 
-from classifier.utils.raster import iterate_rasters_for_data
-from classifier.utils.general import write_tifs
 from classifier.settings import US_ALGORITHMS
 from classifier.timeseries import get_timeseries_samples
+from classifier.utils.general import write_tifs
+from classifier.utils.raster import iterate_rasters_for_data
+from classifier.utils.config import Configuration
 
 # need to set the temp dir for multiprocessing in a container
 process.current_process()._config['tempdir'] = '/tmp/'
@@ -20,7 +21,7 @@ process.current_process()._config['tempdir'] = '/tmp/'
 PREDICT_LOGGER = logging.getLogger(__name__)
 
 
-def prediction_class_probabilities(predicted, valid, meta, config_dict, window):
+def prediction_class_probabilities(predicted, valid, meta, config, window):
     """Gets probability for the classified class and writes the tif for a
     windows
 
@@ -35,17 +36,18 @@ def prediction_class_probabilities(predicted, valid, meta, config_dict, window):
                              .astype(np.float32))
     result_proba = np.full(valid.shape, -9999, np.float32)
     result_proba[valid] = labels_internal_proba
-    write_tifs(os.path.join(config_dict['tmp_dir'], 'probability'),
+    write_tifs(os.path.join(config.tmp_dir, 'probability'),
                window,
                meta['dst_proba_meta'],
                result_proba)
 
+
 def prediction_all_probabilities(
         predicted: array,
         valid: array,
         labels: dict,
         meta: dict,
-        config_dict: dict,
+        config: Configuration,
         window: Window):
     """Gets probability for the classified class and writes the tif for a
     windows
@@ -64,12 +66,13 @@ def prediction_all_probabilities(
         result_proba = np.full(valid.shape, -9999, np.float32)
         result_proba[valid] = probabilities
         write_tifs(
-            os.path.join(config_dict['tmp_dir'], str(labels[internal_label])),
+            os.path.join(config.tmp_dir, str(labels[internal_label])),
             window,
             meta['dst_proba_meta'],
             result_proba)
 
-def do_prediction_proba_su(data, valid, model_dict, config_dict, meta, window):
+
+def do_prediction_proba_su(data, valid, model_dict, config, meta, window):
     """Does the actual prediction and probability description for supervised
     methods
 
@@ -77,7 +80,7 @@ def do_prediction_proba_su(data, valid, model_dict, config_dict, meta, window):
         data: Numpy array with features
         valid: Which parts of the numpy array have valid data
             model_dict:  The model dictionary containting algorithm info
-            config_dict: The cli configuration
+            config: The cli configuration
             meta: The rasterio meta dictionary for the outputs
             window: The rasterio Window for which to do the prediction
 
@@ -101,27 +104,28 @@ def do_prediction_proba_su(data, valid, model_dict, config_dict, meta, window):
     result[valid] = labels_transformed
 
     # See if we need to output some of the modelled probabilities
-    if config_dict['su_probability']:
+    if config.supervised.probability:
         prediction_class_probabilities(
             predicted,
             valid,
             meta,
-            config_dict,
+            config,
             window
         )
-    if config_dict['su_all_probabilities']:
+    if config.supervised.all_probabilities:
         prediction_all_probabilities(
             predicted,
             valid,
             model_dict['labels'],
             meta,
-            config_dict,
+            config,
             window
         )
 
     return result
 
-def do_prediction_proba(data, valid, model_dict, config_dict, meta, window):
+
+def do_prediction_proba(data, valid, model_dict, config, meta, window):
     """Calls the actual probability functions for (un)supervised
     methods
 
@@ -129,7 +133,7 @@ def do_prediction_proba(data, valid, model_dict, config_dict, meta, window):
         data: Numpy array with features
         valid: Which parts of the numpy array have valid data
             model_dict:  The model dictionary containting algorithm info
-            config_dict: The cli configuration
+            config: The cli configuration
             meta: The rasterio meta dictionary for the outputs
             window: The rasterio Window for which to do the prediction
 
@@ -137,12 +141,12 @@ def do_prediction_proba(data, valid, model_dict, config_dict, meta, window):
         The probability array
 
     """
-    if config_dict['app_algorithm'] == 'singleclass':
+    if config.app.algorithm == 'singleclass':
         result = predict_single_class(
             data,
             valid,
             model_dict,
-            config_dict,
+            config,
             meta,
             window
         )
@@ -150,12 +154,12 @@ def do_prediction_proba(data, valid, model_dict, config_dict, meta, window):
     return do_prediction_proba_su(data,
                                   valid,
                                   model_dict,
-                                  config_dict,
+                                  config,
                                   meta,
                                   window)
 
 
-def gather_data_for_prediction(rasters, meta, config_dict, window):
+def gather_data_for_prediction(rasters, meta, config, window):
     """ Gather all the data from the input rasters and return validity raster
 
     Args:
@@ -186,14 +190,15 @@ def gather_data_for_prediction(rasters, meta, config_dict, window):
             f_data,
             data_array,
             rasters,
-            config_dict
+            config
         )
         return data, valid
 
+
 def gather_data_for_prediction_ts(
         rasters: list,
         meta: dict,
-        config_dict:dict,
+        config: Configuration,
         window: Window):
     """Gathers all the data neccessary for a prediction using a raster TS
 
@@ -222,19 +227,19 @@ def gather_data_for_prediction_ts(
 
     rois = {'properties': {'id': 1},
             'geometry': {'coordinates': window_polygon,
-                         'type':'Polygon'}}
+                         'type': 'Polygon'}}
 
     pixel_data = get_timeseries_samples(rasters,
                                         rois,
                                         out_dir=None,
-                                        config_dict=config_dict)
+                                        config=config)
     data = pixel_data.transpose().values
     # All TS is imputed, so we assume that all data is valid.
-    valid = np.ones(shape[1:], dtype=np.bool)
+    valid = np.ones(shape[1:], dtype=bool)
     return data, valid
 
 
-def prediction(window, rasters, model_dict, meta, config_dict):
+def prediction(window, rasters, model_dict, meta, config):
     """ Prediction for a window and save to a tmp file
 
     Args:
@@ -242,24 +247,24 @@ def prediction(window, rasters, model_dict, meta, config_dict):
         rasters  The input raster list
         model_dict Dictionary specifying the algorithm, model labels
         meta The meta files from source, destination and dest_proba
-        config_dict Command line arguments
+        config (Configuration) contains config
     """
-    if config_dict['app_rasters_are_timeseries']:
+    if config.app.rasters_are_timeseries:
         data, valid = \
-            gather_data_for_prediction_ts(rasters, meta, config_dict,  window)
+            gather_data_for_prediction_ts(rasters, meta, config,  window)
     else:
         data, valid = \
-            gather_data_for_prediction(rasters, meta, config_dict, window)
+            gather_data_for_prediction(rasters, meta, config, window)
     if not valid.any():
         return
 
-    if not config_dict['app_algorithm'] in \
-           US_ALGORITHMS:
+    if not config.app.algorithm in \
+            US_ALGORITHMS:
         result = do_prediction_proba(
             data,
             valid,
             model_dict,
-            config_dict,
+            config,
             meta,
             window
         )
@@ -269,7 +274,7 @@ def prediction(window, rasters, model_dict, meta, config_dict):
         result[valid] = labels_external
 
     # Write the classification results to tifs
-    write_tifs(os.path.join(config_dict['tmp_dir'], 'classification'),
+    write_tifs(os.path.join(config.tmp_dir, 'classification'),
                window,
                meta['dst_meta'],
                result)
@@ -278,7 +283,7 @@ def prediction(window, rasters, model_dict, meta, config_dict):
 def predict_proba_single_class(data,
                                valid,
                                model_dict,
-                               config_dict,
+                               config,
                                meta,
                                window):
     """
@@ -301,16 +306,16 @@ def predict_proba_single_class(data,
     result_proba = result.copy().astype(np.float32)
     result_proba[valid] = labels_internal_highest_p
 
-    write_tifs(os.path.join(config_dict['tmp_dir'], 'probability'),
+    write_tifs(os.path.join(config.tmp_dir, 'probability'),
                window,
-               meta,
+               meta["dst_proba_meta"],
                result_proba)
 
 
 def predict_single_class(data,
                          valid,
                          model_dict,
-                         config_dict,
+                         config,
                          meta,
                          window):
     """
@@ -319,7 +324,7 @@ def predict_single_class(data,
         data: input data (array)
         valid: where input data is valid (array)
         model_dict: Model dictionary containing algorithm info
-        config_dict: Configuration dictionary
+        config: Configuration dictionary
         meta: rasterio output meta dictionary
         window: rasterio window
 
@@ -333,11 +338,11 @@ def predict_single_class(data,
     result = np.full(valid.shape, -9999, np.int32)
 
     # See if we need to output some of the modelled probabilities
-    if config_dict['su_probability']:
+    if config.supervised.probability:
         predict_proba_single_class(data,
                                    valid,
                                    model_dict,
-                                   config_dict,
+                                   config,
                                    meta,
                                    window)
     result[valid] = predicted
