@@ -1,5 +1,6 @@
 """General utilities"""
 import contextlib
+import dataclasses
 import inspect
 import json
 import logging
@@ -8,21 +9,24 @@ import pickle
 import shutil
 import sys
 import tempfile
-from typing import List, Callable
+from pathlib import Path
+from typing import Callable, List
 from uuid import uuid4
 from zipfile import ZipFile
 
 import joblib
-from joblib import Parallel, delayed
 import numpy as np
 import rasterio
+from joblib import Parallel, delayed
 from sklearn.impute import SimpleImputer
 from tqdm import tqdm
 
-from classifier.settings import WORKSPACE, PARAMETERS, RASTER_EXTENSIONS
+from classifier.settings import PARAMETERS, RASTER_EXTENSIONS, WORKSPACE
+from classifier.utils.config import setup_config
 
 UTILS_GENERAL_LOGGER = logging.getLogger(__name__)
 
+
 def get_available_model_args(model_args, function):
     """Gets the available arguments for the specific function
         Args:
@@ -56,8 +60,7 @@ def progress(count, total, status=''):
     sys.stdout.flush()
 
 
-
-def cli_init(output_name, rasters, overwrite, config_location, rois= None):
+def cli_init(output_name, rasters, overwrite, config_location, rois=None):
     """Initialize everything before starting real work.
 
     Args:
@@ -79,28 +82,46 @@ def cli_init(output_name, rasters, overwrite, config_location, rois= None):
     np.random.seed(0)
 
     rasters = get_raster_paths(WORKSPACE, rasters)
-    config_dict = read_config(config_location)
+    config = read_config(config_location)
 
-    output_directory = create_output_dir(WORKSPACE,
-                                         output_name,
-                                         overwrite)
+    output_directory = create_output_dir(WORKSPACE, output_name, overwrite)
 
-    config_dict['tmp_dir'] = os.path.join(output_directory, 'tmp')
-    os.mkdir(config_dict['tmp_dir'])
-    init_logger(output_directory, config_dict['app_log_level'])
-    params_str = '\n'.join(
-        [
-            f'{key} :  {value}' for key, value in sorted(config_dict.items())
-        ]
-    )
+    config.tmp_dir = Path(output_directory) / 'tmp'
+    config.tmp_dir.mkdir()
+    init_logger(output_directory, config.app.log_level)
+    params_str = config_as_str(config)
     UTILS_GENERAL_LOGGER.info(
         "\nRunning the Classifier with the following parameters:"
         "\n  %s", params_str)
     if rois is not None:
         rois = os.path.join(WORKSPACE, rois)
-    return output_directory, rasters, rois, config_dict
+    return output_directory, rasters, rois, config
 
 
+def config_as_str(config):
+    """ Adds config values to string
+    Args:
+        config (Configuration) contains config
+
+    Returns:
+        params_str (str): parameters formatted in a string
+    """
+    params_str = ''
+    for key, value in dataclasses.asdict(config).items():
+        if isinstance(value, dict):
+            params_str = params_str + f'{key}:\n'
+            for key_2, value_2 in value.items():
+                if isinstance(value_2, dict):
+                    params_str = params_str + f'    {key_2}:\n'
+                    for key_3, value_3 in value_2.items():
+                        params_str = params_str + \
+                            f'        {key_3}:  {value_3}\n'
+                else:
+                    params_str = params_str + f'    {key_2}:  {value_2}\n'
+        else:
+            params_str = params_str + f'{key}:  {value}\n'
+    return params_str
+
 
 def create_output_dir(workspace, name, overwrite):
     """Create output directory.
@@ -117,8 +138,9 @@ def create_output_dir(workspace, name, overwrite):
     """
     if name is None:
         name = str(uuid4())[:6]
-        UTILS_GENERAL_LOGGER.info("No name argument found. Making new folder "
-                                  "called: %s", name)
+        UTILS_GENERAL_LOGGER.info(
+            "No name argument found. Making new folder "
+            "called: %s", name)
     output_directory = os.path.join(workspace, name)
     if os.path.exists(output_directory):
         if overwrite:
@@ -136,43 +158,71 @@ def create_output_dir(workspace, name, overwrite):
     return output_directory
 
 
-def save_dict_as_json(out_file=None, dict_to_save=None):
-    """Save default config values.
-
-    Save default values for classifier parameters in json in current workspace.
+def save_dict_as_json(out_file, dict_to_save):
+    """Saves a dict to json
 
     Args:
         out_file (str): file to write
         dict_to_save (dict): Dictionary to save
 
     """
-    if out_file is None:
-        out_file = os.path.join(WORKSPACE, 'config.json')
-        dict_to_save = PARAMETERS
     with open(out_file, 'w', encoding='UTF-8') as config:
         config.write(json.dumps(dict_to_save,
                                 sort_keys=True,
-                                indent=1))
+                                indent=1,
+                                default=str))
+
+
+def save_config_as_json(**kwargs):
+    """Save default config values.
+
+    Save default values for classifier parameters in json in current workspace.
+
+    Args:
+        kwargs (dict): values to put into the config
+
+    """
+    out_file = Path(WORKSPACE) / 'config.json'
+    dict_to_save = PARAMETERS
+    # update dict with provided parameters
+    dict_to_save = update_dict(dict_to_save, **kwargs)
+    with open(out_file, 'w', encoding='UTF-8') as config:
+        config.write(json.dumps(dict_to_save, indent=4, default=str))
+
+
+def update_dict(dict_to_update: dict, **kwargs):
+    """Update dict with provided values
+
+    Args:
+        dict_to_update (dict): dict which values should be updated
+        **kwargs (dict): keywords and values to update
+
+    Returns:
+        updated_dict (dict): updated dict
+    """
+    updated_dict = {}
+    for key, value in dict_to_update.items():
+        if isinstance(value, dict):
+            updated_dict[key] = update_dict(value, **kwargs)
+        else:
+            updated_dict[key] = kwargs.get(key, value)
+    return updated_dict
 
 
 def read_config(config_file):
     """Read config parameter file and change defaults where necessary.
 
     Args:
-        config_file (str): location of the config file
-    returns:
-        PARAMETERS (dict): dictionary with parameters
+        config_file (pathlib.Path): location of the config file
+
+    Returns:
+        config (Configuration) nested dataclass with all config values
     """
-    config_file = os.path.join(WORKSPACE, config_file)
-    if os.path.isfile(config_file):
-        with open(config_file, encoding='UTF-8') as json_config:
-            config_dict = json.load(json_config)
-        for key in config_dict.keys():
-            PARAMETERS[key] = config_dict[key]
-    else:
-        UTILS_GENERAL_LOGGER.warning(
-            "No local config file found. Using defaults")
-    return PARAMETERS
+    config_path = Path(WORKSPACE) / config_file
+
+    config = setup_config(config_path)
+
+    return config
 
 
 def init_logger(output_directory, log_level):
@@ -188,6 +238,9 @@ def init_logger(output_directory, log_level):
     logger = logging.getLogger(__name__)
     logger.setLevel(logging.DEBUG)
 
+    # disable matplotlib logger
+    logging.getLogger('matplotlib.font_manager').disabled = True
+
     # # create console handler and set level to info
     console_handler = logging.StreamHandler(sys.stdout)
     console_handler.setLevel(log_level)
@@ -200,14 +253,14 @@ def init_logger(output_directory, log_level):
     logging.getLogger('').addHandler(console_handler)
 
 
-def impute_values(dataset, args):
+def impute_values(dataset, config):
     """Impute values
 
     Uses the sklearn SimpleImputer to impute missing values.
 
     Args:
         dataset (DataFrame): DataFrame containing Nans
-        args (dict): Parameters to use for imputation
+        config (Configuration) Parameters to use for imputation
 
     returns:
         dataset (DataFrame): DataFrame with Nans imputed
@@ -217,22 +270,23 @@ def impute_values(dataset, args):
         dataset = np.where(np.isinf(dataset), np.nan, dataset)
     else:
         dataset = dataset.replace([np.inf, -np.inf], np.nan)
-    verbosity_level = 0 if args['app_log_level'] == 'INFO' else 2
-    if np.isnan(dataset).all() and not args['app_imputation_strategy'] == \
-                                       'constant':
+    verbosity_level = 0 if config.app.log_level == 'INFO' else 2
+    if np.isnan(dataset).all() and not config.app.imputation_strategy == \
+            'constant':
         UTILS_GENERAL_LOGGER.warning(
             "Empty window found, Falling back to constant "
             "strategy")
         strategy = 'constant'
     else:
-        strategy = args['app_imputation_strategy']
+        strategy = config.app.imputation_strategy
 
     imputer = SimpleImputer(strategy=strategy,
-                            fill_value=args['app_imputation_constant'],
+                            fill_value=config.app.imputation_constant,
                             verbose=verbosity_level)
 
     return imputer.fit_transform(dataset)
 
+
 def write_tifs(temp_dir, window, meta, result):
     """
     Writes the tifs of the individual windows
@@ -250,22 +304,17 @@ def write_tifs(temp_dir, window, meta, result):
     os.makedirs(temp_dir, exist_ok=True)
     meta['compress'] = 'deflate'
     with rasterio.open(
-            os.path.join(
-                temp_dir,
-                f"c{window.col_off}_{window.row_off}.tif"
-            ),
-            'w',
-            **meta
-    ) as dst:
+            os.path.join(temp_dir, f"c{window.col_off}_{window.row_off}.tif"),
+            'w', **meta) as dst:
         dst.write_band(1, result)
 
 
-def impute(data_array, config_dict):
+def impute(data_array, config):
     """Impute missing values for prediction
 
     Args:
         data_array (Numpy Array): Array containing missing values
-        config_dict (dict): dictionary with all application parameters
+        config (Configuration): dictionary with all application parameters
 
     returns:
         Imputed data_array
@@ -273,25 +322,25 @@ def impute(data_array, config_dict):
     """
     shape = data_array.shape
     reshaped_array = data_array.reshape(shape[0], (shape[1] * shape[2])).T
-    imputed_array = impute_values(reshaped_array, config_dict)
+    imputed_array = impute_values(reshaped_array, config)
     imp_shape = imputed_array.shape
     if not shape[0] * shape[1] * shape[2] == imp_shape[0] * imp_shape[1]:
         UTILS_GENERAL_LOGGER.warning("Not all columns could be calculated. "
                                      "Falling  back to constant strategy for "
                                      "this chunk")
-        config_dict['app_imputation_strategy'] = 'constant'
-        imputed_array = impute_values(reshaped_array, config_dict)
+        config.app.imputation_strategy = 'constant'
+        imputed_array = impute_values(reshaped_array, config)
     imputed_array = imputed_array.T.reshape(shape[0], shape[1], shape[2])
     return imputed_array
 
 
-def save_model(model_dict, out_dir, config_dict):
+def save_model(model_dict, out_dir, config):
     """Save the model as a pickle  and metadata  as a json and zip
 
         Args:
             model_dict (dict):  Contains the model as well as metadata
             out_dir (str):      Path where to write model file and meta
-            config_dict (dict): Configuration parameters
+            config (Configuration): Configuration parameters
 
     """
     to_save = model_dict.copy()
@@ -308,11 +357,11 @@ def save_model(model_dict, out_dir, config_dict):
     # Save meta as json
     save_dict_as_json(meta_tmp, to_save)
     to_write = {
-        meta_tmp: f"{config_dict['name']}_meta.json",
-        pickle_tmp: f"{config_dict['name']}.model"
+        meta_tmp: f"{config.name}_meta.json",
+        pickle_tmp: f"{config.name}.model"
     }
 
-   # Zip to output directory
+    # Zip to output directory
     zipfile_path = os.path.join(out_dir, 'model.zip')
     write_zipfile(to_write, zipfile_path)
 
@@ -334,6 +383,7 @@ def write_zipfile(files_to_write, zipfile_path):
             print(files, files_to_write[files])
             model_zip.write(files, files_to_write[files])
 
+
 def get_raster_paths(workspace, rasters):
     """Get full paths for all rasters.
 
@@ -350,11 +400,7 @@ def get_raster_paths(workspace, rasters):
     for raster in rasters:
         path = os.path.join(workspace, raster)
         if os.path.isdir(path):
-            paths += [
-                os.path.join(path, r)
-                for r
-                in sorted(os.listdir(path))
-            ]
+            paths += [os.path.join(path, r) for r in sorted(os.listdir(path))]
         else:
             paths.append(path)
     return [x for x in paths if os.path.splitext(x)[-1] in RASTER_EXTENSIONS]
@@ -375,14 +421,11 @@ def unzip_model_file(model_file):
         zipf.extractall(tmpdir)
 
     model_files = os.listdir(tmpdir)
-    meta_file = os.path.join(
-        tmpdir,
-        [x for x in model_files if x.endswith('.json')][0]
-    )
+    meta_file = os.path.join(tmpdir,
+                             [x for x in model_files
+                              if x.endswith('.json')][0])
     pickle_file = os.path.join(
-        tmpdir,
-        [x for x in model_files if x.endswith('.model')][0]
-    )
+        tmpdir, [x for x in model_files if x.endswith('.model')][0])
     return meta_file, pickle_file
 
 
@@ -402,8 +445,8 @@ def read_model(model_file):
     with open(meta_file, encoding='UTF-8') as json_config:
         model_dict = json.load(json_config)
 
-    params_str = '\n'.join([f'{key} :  {value}'for key, value in
-                            sorted(model_dict.items())])
+    params_str = '\n'.join(
+        [f'{key} :  {value}' for key, value in sorted(model_dict.items())])
 
     with open(pickle_file, 'rb') as pfile:
         model_dict['model'] = pickle.load(pfile)
@@ -414,7 +457,9 @@ def read_model(model_file):
 
     return model_dict
 
-def parallel_function(func: Callable, iterable: List[dict],
+
+def parallel_function(func: Callable,
+                      iterable: List[dict],
                       ncpus: int = 1) -> list:
     """Runs a function in parallel using Joblib
         Args:
@@ -424,11 +469,8 @@ def parallel_function(func: Callable, iterable: List[dict],
         Returns:
             list of results
     """
-    with tqdm_joblib(
-            tqdm(total=len(iterable))
-    ) as _:
-        result = Parallel(
-            n_jobs=ncpus)(delayed(func)(**x) for x in iterable)
+    with tqdm_joblib(tqdm(total=len(iterable))) as _:
+        result = Parallel(n_jobs=ncpus)(delayed(func)(**x) for x in iterable)
 
     return result
 
