@@ -5,6 +5,7 @@ import os
 import re
 import subprocess
 import sys
+
 import numpy as np
 import rasterio
 from rasterio.enums import Resampling
@@ -12,7 +13,6 @@ from rasterio.vrt import WarpedVRT
 
 from classifier.utils.general import impute
 
-
 UTILS_RASTER_LOGGER = logging.getLogger(__name__)
 
 
@@ -51,18 +51,19 @@ def stitch_(out_ds, temp_dir, meta, dtype='Int32'):
     os.remove(out_ds+'.vrt')
 
 
-def stitch(out_prefix, meta, config_dict):
+def stitch(out_prefix, meta, config):
     """
      Stitch the temporary files together
     Args:
         out_prefix (str): The output directory
         meta (dict): The rasterio meta dicts from source, destination and
                      dest_proba
+        config (Configuration) contains config
 
 
     """
     # Find all directories that have chunks.
-    dirs = os.listdir(config_dict['tmp_dir'])
+    dirs = os.listdir(config.tmp_dir)
     for directories in dirs:
         if directories.startswith('class'):
             out_file = os.path.join(out_prefix, 'classification')
@@ -75,15 +76,14 @@ def stitch(out_prefix, meta, config_dict):
             dtype = 'Float32'
         UTILS_RASTER_LOGGER.info("Now stitching %s", out_file)
         stitch_(out_file,
-                os.path.join(config_dict['tmp_dir'], directories),
+                os.path.join(config.tmp_dir, directories),
                 meta,
                 dtype)
 
 
-
 def iterate_rasters_for_data(
         window, first, f_data,
-        data_array, rasters, config_dict):
+        data_array, rasters, config):
     """
     Iterate over all rasters to add data into a single array
     Args:
@@ -92,6 +92,7 @@ def iterate_rasters_for_data(
         f_data: the data from the first raster
         data_array: the array where all raster data is aggregated
         rasters: list of rasters
+        config (Configuration) contains config
 
 
     Returns:
@@ -123,8 +124,8 @@ def iterate_rasters_for_data(
     data_array[data_array == first.nodatavals[0]] = np.nan
     if isinstance(data_array, list):
         data_array = np.concatenate(data_array, axis=0)
-    if config_dict['app_imputation'] and data_array.size > 0:
-        data_array = impute(data_array, config_dict)
+    if config.app.imputation and data_array.size > 0:
+        data_array = impute(data_array, config)
     valid = np.isfinite(data_array).all(axis=0)
     data = data_array[:, valid].T
     return data, valid
@@ -203,6 +204,7 @@ def get_raster_date(raster):
     file_date = re.findall(r'\d\d\d\d-\d\d-\d\d', raster)[0]
     return file_date
 
+
 def raster_warp_dst(raster):
     """Get the warp paramters of a raster:
 
@@ -221,6 +223,7 @@ def raster_warp_dst(raster):
         }
     return warp_dst
 
+
 def count_bands(raster):
     """Returns number of bands of a raster
 
@@ -231,6 +234,7 @@ def count_bands(raster):
     with rasterio.open(raster, 'r') as src:
         return src.count
 
+
 def verify_and_count_bands(rasters):
     """Counts the number of bands for all the rasters and makes sure they all
     have the same number bands. Then it returns the number of bands that the
